{% extends "!page.html" %}

{% block right_sidebar %}

<!-- TOC -->
<div>
  <div style="text-align: center; font-size: small; margin: 5px; color: gray">Advertisement</div>
  <script type="text/javascript" src="https://udbaa.com/bnr.php?section=TOC&pub=765231&format=300x250&ga=g&bg=1"></script>
  <noscript><a href="https://yllix.com/publishers/765231" target="_blank"><img src="//ylx-aff.advertica-cdn.com/pub/300x250.png" style="border:none;margin:0;padding:0;vertical-align:baseline;" alt="ylliX - Online Advertising Network" /></a></noscript>
</div>

{{ super() }}
{% endblock right_sidebar %}

{% block footer %}

{{ super() }}

<div>
  <div style="text-align: center; font-size: small; margin: 5px; color: gray">Advertisement</div>
  <script type="text/javascript" src="https://udbaa.com/bnr.php?section=Bottom&pub=765231&format=728x90&ga=g&bg=1"></script>
  <noscript><a href="https://yllix.com/publishers/765231" target="_blank"><img src="//ylx-aff.advertica-cdn.com/pub/728x90.png" style="border:none;margin:0;padding:0;vertical-align:baseline;" alt="ylliX - Online Advertising Network" /></a></noscript>
</div>

{% endblock footer %}
